<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * @package PaymentModuleEPNCC
 * @author Egor V. Derevyankin
 */

class Payment_Module_EPN_CC extends pm_sm_api
{

    /* static */ function getInitialCurrencySettings()
    {
        $res = array
        (
            'currency_acceptance_rules' => array
            (
                array
                (
                    'rule_name'     =>  ACTIVE_AND_ACCEPTED_BY_PM_SM_AND_SELECTED_BY_CUSTOMER
                   ,'rule_selected' =>  DB_FALSE
                )
               ,array
                (
                    'rule_name'     =>  THE_ONLY_ACCEPTED
                   ,'rule_selected' =>  DB_TRUE
                )
               ,array
                (
                    'rule_name'     =>  MAIN_STORE_CURRENCY
                   ,'rule_selected' =>  DB_FALSE
                )
            )

            /**
             * Website Payments Standard Integration Guide, Last updated: March 2008.
             * TABLE 2.6 Currencies Allowed for Transactions and Balances, Currency ISO-4217 Code
             */
           ,'accepted_currencies' => array
            (
                array
                (
                    'currency_code'   => 'AUD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CAD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CHF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'CZK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'DKK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'EUR'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'GBP'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HKD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'HUF'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'JPY'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NOK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'NZD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'PLN'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SEK'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'SGD'
                   ,'currency_status' => 'NOT_ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'USD'
                   ,'currency_status' => 'THE_ONE_ONLY_ACCEPTED'
                )
            )
        );
        return $res;
    }

    function Payment_Module_EPN_CC()
    {}

    function install()
    {
        global $application;
        loadCoreFile('csv_parser.php');
        $csv_parser = new CSV_Parser();

        $tables = Payment_Module_EPN_CC::getTables();
        $query = new DB_Table_Create($tables);

        $table = 'pm_epn_settings';
        $columns = $tables[$table]['columns'];

        list($flt,$Default_Settings) = $csv_parser->parse_file(dirname(__FILE__)."/includes/default_settings.csv");
        if(count($Default_Settings) > 0)
        {
            foreach($Default_Settings as $key => $setting)
            {
                $query = new DB_Insert($table);
                $query->addInsertValue($setting["key"], $columns['setting_key']);
                $query->addInsertValue($setting["value"], $columns['setting_value']);
                $application->db->getDB_Result($query);
            };
        };

        //                                RemovePersonInfoType
        //                   ,          -     CheckoutEditor
        //             PersonInfoType,         , BillingInfo, ShippingInfo,
        //  CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
        //             ), ShippingMethodInfo (              ).
        //       PaymentModule'                                                 ,
        //                                     .
        //                    ,                 PersonInfoType          .
        //
        //  PersonInfoType.
        modApiFunc('EventsManager',
               'addEventHandler',
               'RemovePersonInfoTypeEvent',
               'Payment_Module_EPN_CC',
               'OnRemovePersonInfoType');
    }

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
     *            ), ShippingMethodInfo (              ).
     *      PaymentModule'                                                 ,
     *                                    .
     *                   ,                 PersonInfoType          .
     *
     * PersonInfoType.
     *
     *        Payment_Module_EPN_CC                    2 PersonInfoType' :
     * billingInfo
     * creditCardInfo
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if($this->isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this->MessageResources = &$application->getInstance('MessageResources',"payment-module-epn-messages", "AdminZone");
            $msg = $this->MessageResources->getMessage('MODULE_PAYMENT_EPN_API_MSG_REMOVE_PERSON_INFO_TYPE');

            $value = NULL;
            switch($person_info_type_tag)
            {
                case "billingInfo":
                {
                    $value = $msg;
                    break;
                }
                case "creditCardInfo":
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }

    function uninstall()
    {
        $query = new DB_Table_Delete(Payment_Module_EPN_CC::getTables());
    }

    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array();

        $table = 'pm_epn_settings';
        $tables[$table] = array(
            'columns' => array(
                'setting_id'    => $table.'.setting_id'
               ,'setting_key'   => $table.'.setting_key'
               ,'setting_value' => $table.'.setting_value'
             )
           ,'types'   => array(
                'setting_id'    => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'setting_key'   => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
               ,'setting_value' => DBQUERY_FIELD_TYPE_CHAR255.' NOT NULL'
             )
           ,'primary' => array(
                'setting_id'
             )
        );

        global $application;
        return $application->addTablePrefix($tables);
    }

    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules","payment");
        return array_key_exists($this->getUid(), $active_modules);
    }

    function getUID()
    {
//        return '00F3E097-BE1A-482F-9BD4-73E00CF8A107';
        include(dirname(__FILE__)."/includes/uid.php");
        return $uid;
    }

    function getInfo()
    {
        global $application;

        $MR = &$application->getInstance('MessageResources','payment-module-epn-messages','AdminZone');

        return array("GlobalUniquePaymentModuleID" => $this->getUid(),
                     "Name"        => $MR->getMessage('MODULE_NAME'),
                     "Description" => $MR->getMessage('MODULE_DESCR'),
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$MR->getMessage('MODULE_STATUS_ACTIVE')."</span>":$MR->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>",
                     "PreferencesAZViewClassName" => "CheckoutPaymentModuleEPNCCInputAZ",
                     "CZInputViewClassName" => "CheckoutPaymentModuleEPNCCInput",
                     "CZOutputViewClassName" => "CheckoutPaymentModuleEPNCCOutput",
                     "APIClassName" => __CLASS__
                    );
    }

    function getSettings()
    {
        $res = execQuery('SELECT_PM_SM_SETTINGS2',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'pm_epn_settings'));
        $settings=array();

        foreach($res as $k => $sval)
            $settings[$sval['setting_key']]=$sval['setting_value'];

        return $settings;
    }

    function updateSettings($settings)
    {
        global $application;
        $tables=$this->getTables();
        $stable=$tables['pm_epn_settings']['columns'];

        foreach($settings as $skey => $sval)
        {
            $query = new DB_Update('pm_epn_settings');
            $query->addUpdateValue($stable['setting_value'],$sval);
            $query->WhereValue($stable['setting_key'], DB_EQ, $skey);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        return;
    }

    function getPaymentMode()
    {
        return "direct";
    }

    function getAdditionalPersonInfoVariantTag($prerequisite_type)
    {
        if($prerequisite_type == "creditCardInfo")
        {
            return "default";
        }
        else
        {
            return false;
        }
    }

    function export_cc_month($month)
    {
    	return sprintf("%02d", (int)$month);
    }

    function export_cc_year($year)
    {
        return sprintf("%02d", ((int)$year) % 100);
    }

    function backgroundProcess($type="direct", $transactionData = null)
    {
        global $application;
        loadCoreFile('aal.class.php');

        $settings = $this->getSettings();

        if($settings['MODULE_MODE'] == EPN_MODE_TEST)
        {
            $epn = &$application->getInstance('EpnPayment', EPN_DEFAULT_ACCOUNT_NUMBER, EPN_TEST_GATEWAY_URL);
        }
        else
        {
            $epn = &$application->getInstance('EpnPayment', $settings['EPN_ACCOUNT_NUMBER'], EPN_DEFAULT_GATEWAY_URL);
        };

        $billingInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getPrerequisiteValidationResults", "billingInfo"));
        $billingInfo->setAccessMask("validatedData", AAL_CUSTOM_PARAM, "value");

        $CCInfo = new ArrayAccessLayer(modApiFunc("Checkout", "getDecryptedCreditCardInfoPrerequisiteData", $this->getUid()));
        $CCInfo->setAccessMask(AAL_CUSTOM_PARAM, "value");

        //$epn->setEPNAccountNumber( $EPNAccountNumber );
        if($settings['MODULE_MODE'] == EPN_MODE_TEST)
        {
            $epn->setCardNo( $settings['EPN_DEFAULT_CARD_NO'] );
        }
        else
        {
            $epn->setCardNo($CCInfo->getByMask('CreditCardNumber'));
        }
        $epn->setExpMonth($this->export_cc_month($CCInfo->getByMask('ExpirationMonth')) );
        $epn->setExpYear($this->export_cc_year($CCInfo->getByMask('ExpirationYear')) );
        $epn->setTotal( $this->export_PRICE_N_A(modApiFunc("Checkout", "getOrderPrice", "Total", modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", ORDER_NOT_CREATED_YET, $this->getUid())) ) );
        $epn->setAddress( $billingInfo->getByMask('Streetline1') . ' ' . $billingInfo->getByMask('Streetline2') );
        $epn->setZip( $billingInfo->getByMask('Postcode') );
        $epn->setEMail( $billingInfo->getByMask('Email') );
        $cvv2 = $CCInfo->getByMask('CreditCardVerificationNumber');
        if(empty($cvv2))
        {
	        $epn->setCVV2Type( EPN_CVV2_SHOULD_NOT_BE_USED );
	        $epn->setCVV2( "" );
        }
        else
        {
            $epn->setCVV2Type( EPN_CVV2_SHOULD_BE_USED );
            $epn->setCVV2($cvv2);
        }

        $step_id_to_redirect_to = modApiFunc("Checkout", "getCurrentStepID");
        $res = $epn->doPayment();
        if(isset($res['Error']))
        {
        	//                                                       .
            modApiFunc('Session','set','EPN_ErrorMessage',$res['ErrorMessage']);
            $paymentModule = modApiFunc("Checkout", "getPrerequisiteValidationResults", "paymentModule");
            modApiFunc("Checkout", "setPrerequisitesValidationResultsItem", "paymentModule", "", false, "CHECKOUT_ERR_005", array(), $paymentModule["validatedData"]);
            $step_id_to_redirect_to = modApiFunc('Checkout', 'getStepIDtoRedirectToAfterPrerequisitesValidationErrors', modApiFunc("Checkout", "getCurrentStepID"));
            modApiFunc("Checkout", "saveState");
        }
        else
        {
        	//                                       .
        	//                                     "DECLINED"     "UNABLE".
            //Create order
            $order_id = modApiFunc("Checkout", "createOrderInDB");
            //                .
            //
            modApiFunc("Cart", "removeAllFromCart");

            $MR = &$application->getInstance('MessageResources','payment-module-epn-messages','AdminZone');
            $SMR = &$application->getInstance('MessageResources');

            $txn_status = "";
            $new_order_payment_status = "unchanged";
            switch($res['TrxnStatus'])
            {
            	case EPN_TXN_STATUS_APPROVED:
            		$txn_status = $MR->getMessage("MODULE_PAYMENT_EPN_LOG_MESSAGE_TXN_STATUS_APPROVED");
            		$new_order_payment_status = ORDER_PAYMENT_STATUS_FULLY_PAID;
            		$new_order_payment_status_msg = $SMR->getMessage("ORDER_PAYMENT_STATUS_002");
            		break;
                case EPN_TXN_STATUS_DECLINED:
                    $txn_status = $MR->getMessage("MODULE_PAYMENT_EPN_LOG_MESSAGE_TXN_STATUS_DECLINED");
                    $new_order_payment_status = ORDER_PAYMENT_STATUS_DECLINED;
                    $new_order_payment_status_msg = $SMR->getMessage("ORDER_PAYMENT_STATUS_003");
                    break;
                case EPN_TXN_STATUS_UNABLE:
                    $txn_status = $MR->getMessage("MODULE_PAYMENT_EPN_LOG_MESSAGE_TXN_STATUS_UNABLE");
                    break;
                default:
                    $txn_status = $MR->getMessage("MODULE_PAYMENT_EPN_LOG_MESSAGE_TXN_STATUS_UNKNOWN");
                	break;
            }

            $msg = $MR->getMessage('MODULE_PAYMENT_EPN_LOG_MODULE_ID') . "\n" .
                $this->getUID() . "\n" .
                $this->getmicrotime() . "\n" .
                $settings["MODULE_NAME"] . " ".
                $MR->getMessage("MODULE_PAYMENT_EPN_LOG_EVENT_DESCRIPTION") .
                $MR->getMessage("MODULE_PAYMENT_EPN_LOG_MESSAGE_PASSED") . "\n" .

                $MR->getMessage("MODULE_PAYMENT_EPN_LOG_MESSAGE_TXN_STATUS") . ':' . $txn_status . ':' . $res['TrxnMessage'] . "\n" .
                $MR->getMessage("MODULE_PAYMENT_EPN_LOG_MESSAGE_ADDRESS_STATUS") . ':' . $res['AddressMessage'] . "\n" .
                $MR->getMessage("MODULE_PAYMENT_EPN_LOG_MESSAGE_CVV2_STATUS") . ':' . $res['CVV2Message'] . "\n" .

                $MR->getMessage("MODULE_PAYMENT_EPN_LOG_MESSAGE_FULL_GATEWAY_RESPONSE") . ':' . $res['RawGatewayResponse'] . "\n";

            modApiFunc("Checkout", "addOrderHistory", $order_id, $msg);

            // update order payment status info
            if($new_order_payment_status != "unchanged")
            {
                $msg = $MR->getMessage("MODULE_PAYMENT_EPN_LOG_MESSAGE_ORDER_PAYMENT_STATUS_UPDATED", array($new_order_payment_status_msg));
                modApiFunc("Checkout", "UpdatePaymentStatusInDB", $order_id, $new_order_payment_status, $msg);
            }

            $step_id_to_redirect_to = modApiFunc("Checkout", "getCurrentStepID") + 1;
        }

        $request = new Request();
        $request->setView('CheckoutView');
        $request->setAction("SetCurrStep");
        $request->setKey   ( 'step_id', $step_id_to_redirect_to);
        $application->redirect($request);
    }

    function export_PRICE_N_A($price)
    {
        return ($price == PRICE_N_A) ? 0.0 : $price;
    }

    function getmicrotime()
    {
        global $application;
        list($usec, $sec) = explode(" ", microtime());

        $MR = &$application->getInstance('MessageResources','payment-module-epn-messages','AdminZone');

        return $MR->getMessage('MODULE_PAYMENT_EPN_LOG_EVENT_TIME') .
                (float)($usec + $sec) . " " .
                $MR->getMessage('MODULE_PAYMENT_EPN_SECONDS');
    }

}
?>