<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * @package PaymentModuleIDEALCC
 * @author Egor V. Derevyankin
 *
 */

class Payment_Module_IDEAL_CC extends pm_sm_api
{
    function Payment_Module_IDEAL_CC()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-ideal-messages", "AdminZone");
        $Settings = $this->getSettings();
        $this->OrderHistoryMessageTag = $obj->getMessage('MODULE_PAYMENT_IDEAL_LOG_MODULE_ID') .
                $this->getUID() . "\n" .
                $Settings["MODULE_NAME"];
    }

    /* static */ function getInitialCurrencySettings()
    {
        $res = array
        (
            'currency_acceptance_rules' => array
            (
                array
                (
                    'rule_name'     =>  THE_ONLY_ACCEPTED
                   ,'rule_selected' =>  DB_TRUE
                )
            )
           ,'accepted_currencies' => array
            (
                array
                (
                    'currency_code'   => 'EUR'
                   ,'currency_status' => 'THE_ONE_ONLY_ACCEPTED'
                )
            )
        );
        return $res;
    }


    function install()
    {
        global $application;

        $tables = Payment_Module_IDEAL_CC::getTables();
        $query = new DB_Table_Create($tables);

        loadCoreFile('csv_parser.php');
        $csv_parser = new CSV_Parser();

        $table = 'pm_ideal_settings';
        $columns = $tables[$table]['columns'];

        list($flt,$Default_Settings) = $csv_parser->parse_file(dirname(__FILE__)."/includes/default_settings.csv");
        if(count($Default_Settings) > 0)
        {
            foreach($Default_Settings as $key => $setting)
            {
                $query = new DB_Insert($table);
                $query->addInsertValue($setting["key"], $columns['setting_key']);
                $query->addInsertValue($setting["value"], $columns['setting_value']);
                $application->db->getDB_Result($query);
            };
        };
    }

    function uninstall()
    {
        $query = new DB_Table_Delete(Payment_Module_IDEAL_CC::getTables());
    }

    function getUID()
    {
//        return '2F2B39D5-CF61-4638-8C3D-B6E26250FB3F';
        include(dirname(__FILE__)."/includes/uid.php");
        return $uid;
    }

    function getInfo()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-ideal-messages", "AdminZone");

        $sets = $this->getSettings();

        return array("GlobalUniquePaymentModuleID" => $this->getUid(),
                     "Name"        => $sets["MODULE_NAME"],
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$obj->getMessage('MODULE_STATUS_ACTIVE')."</span>":$obj->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>",
                     "PreferencesAZViewClassName" => "CheckoutPaymentModuleIDEALCCInputAZ",
                     "CZInputViewClassName" => "CheckoutPaymentModuleIDEALCCInput",
                     "CZOutputViewClassName" => "CheckoutPaymentModuleIDEALCCOutput",
                     "APIClassName" => __CLASS__
                    );
    }

    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        };

        $tables = array ();

        $settings = 'pm_ideal_settings';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                => $settings.'.setting_id'
               ,'setting_key'       => $settings.'.setting_key'
               ,'setting_value'     => $settings.'.setting_value'
            );
        $tables[$settings]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'setting_key'       => DBQUERY_FIELD_TYPE_CHAR50
               ,'setting_value'     => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        $table = 'pm_ideal_issuers_cache';
        $tables[$table] = array(
            'columns' => array(
                'issuer_id'     => $table.'.issuer_id'
               ,'issuer_name'   => $table.'.issuer_name'
            )
           ,'types'   => array(
                'issuer_id'     => DBQUERY_FIELD_TYPE_INT . ' NOT NULL'
               ,'issuer_name'   => DBQUERY_FIELD_TYPE_CHAR255 . ' NOT NULL'
            )
           ,'primary' => array(
                'issuer_id'
            )
        );

        global $application;
        return $application->addTablePrefix($tables);
    }

    function getSettings()
    {
        $res = execQuery('SELECT_PM_SM_SETTINGS2',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'pm_ideal_settings'));
        $settings=array();

        foreach($res as $k => $sval)
            $settings[$sval['setting_key']]=$sval['setting_value'];

        return $settings;
    }

    function updateSettings($settings)
    {
        global $application;
        $tables=$this->getTables();
        $stable=$tables['pm_ideal_settings']['columns'];

        foreach($settings as $skey => $sval)
        {
            $query = new DB_Update('pm_ideal_settings');
            $query->addUpdateValue($stable['setting_value'],$sval);
            $query->WhereValue($stable['setting_key'], DB_EQ, $skey);
            $application->db->PrepareSQL($query);
            $application->db->DB_Exec();
        };

        return;
    }

    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules","payment");
        return array_key_exists($this->getUid(), $active_modules);
    }

    function checkRequirments()
    {
        $return=array();

        // check SSL
        loadCoreFile('bouncer.php');
        $bssl = new Bouncer();
        if(!$bssl->isSSLavailable())
            $return[] = "ERROR_SSL_NOT_AVAIL";

        // check OpenSSL extension
        if(!function_exists('openssl_open'))
            $return[] = "ERROR_OPEN_SSL_NOT_AVAIL";

        return $return;
    }

    function getConfirmationData($order_id)
    {
        $trx_data = $this->__processTransactionRequest($order_id);

        //: process null value.

        if(array_key_exists('trx_id',$trx_data))
        {
            modApiFunc('Checkout','updateOrderPaymentProcessorOrderID', $order_id, $trx_data['trx_id']);
        };

        $this->trx_url = $trx_data['url'];

        $confirmationData = array
            (
                 "FormAction" => $trx_data['url']
                ,"FormMethod" => "POST"
                ,"GatewayName" => "iDEAL"
            );

        return $confirmationData;
    }

    function processData($data, $order_id)
    {

        $result = array();
        global $application;
        $msg = &$application->getInstance('MessageResources','payment-module-ideal-messages', 'AdminZone');
        $trx_id = $data['_GET']['trxid'];
        $status = $this->__processStatusRequest($trx_id);

        if($status == 'Success')
        {
            $EventType = 'ConfirmationSuccess';
            $result = modApiFunc(
                'Checkout','UpdatePaymentStatusInDB',$order_id,ORDER_PAYMENT_STATUS_FULLY_PAID,
                $this->OrderHistoryMessageTag . "\n" . $this->getmicrotime() . "\n" .
                $msg->getMessage('MODULE_PAYMENT_IDEAL_LOG_EVENT_DESCRIPTION') .
                $msg->getMessage('MODULE_PAYMENT_IDEAL_LOG_MESSAGE_COMPLETED') . "\n" .
                $msg->getMessage('MODULE_PAYMENT_IDEAL_LOG_EXTRA_STATUS_MESSAGE') . " " .$status);
        }
        else
        {
            $EventType = 'ConfirmationFailure';
            $result = modApiFunc(
                'Checkout','UpdatePaymentStatusInDB',$order_id,ORDER_PAYMENT_STATUS_DECLINED,
                $this->OrderHistoryMessageTag . "\n" . $this->getmicrotime() . "\n" .
                $msg->getMessage('MODULE_PAYMENT_IDEAL_LOG_EVENT_DESCRIPTION') .
                $msg->getMessage('MODULE_PAYMENT_IDEAL_LOG_MESSAGE_FAILED') . "\n" .
                $msg->getMessage('MODULE_PAYMENT_IDEAL_LOG_EXTRA_STATUS_MESSAGE') . " " .$status);
        };

        return array("EventType" => $EventType, "statusChanged" => $result);

    }

    function getOrderIDFromRequest()
    {
        $r = new Request();
        return $r->getValueByKey('ec');
    }

    function export_PRICE_N_A($price)
    {
        return ($price == PRICE_N_A) ? 0.0 : $price;
    }

    function getmicrotime()
    {
        global $application;
        $obj = &$application->getInstance('MessageResources',"payment-module-ideal-messages", "AdminZone");

        list($usec, $sec) = explode(" ", microtime());

        return $obj->getMessage('MODULE_PAYMENT_IDEAL_LOG_EVENT_TIME') .
                (float)($usec + $sec) . " " .
                $obj->getMessage('MODULE_PAYMENT_IDEAL_SECONDS');
    }

    function moveUploadedFileToKeysDir($src_fname)
    {
        $result = array(
            'error' => ''
           ,'full_path' => ''
           ,'base_name' => ''
           ,'file_size' => ''
        );

        if(!array_key_exists($src_fname,$_FILES))
        {
            $result['error'] = UPLOAD_ERR_NO_FILE;
            return $result;
        };

        $src_file = $_FILES[$src_fname];
        $result['error'] = $src_file['error'];

        if($result['error'] != UPLOAD_ERR_OK)
        {
            return $result;
        };

        if(!is_uploaded_file($src_file['tmp_name']))
        {
            $result['error'] = UPLOAD_ERR_POSIBLE_ATTACK;
            return $result;
        };

        $dest_filepath = PM_IDEAL_KEYS_DIR._ml_strtolower($src_fname);

        if(!move_uploaded_file($src_file['tmp_name'],$dest_filepath))
        {
            $result['error'] = UPLOAD_ERR_CANT_MOVE_FILE;
            return $result;
        };

        $result['full_path'] = $dest_filepath;
        $result['base_name'] = basename($dest_filepath);
        $result['file_size'] = $src_file['size'];

        return $result;
    }

    function checkPrivateKeyAndCert($key_file, $cert_file, $key_passwd)
    {
        $private_key = openssl_pkey_get_private(file_get_contents($key_file), $key_passwd);

        if($private_key === false)
        {
            return false;
        };

        $public_key = openssl_x509_read(file_get_contents($cert_file));

        if($public_key === false)
        {
            return false;
        };

        if(!openssl_x509_check_private_key($public_key, $private_key))
        {
            return false;
        };

        return true;
    }

    function areCurrentKeysCorrect()
    {
        $settings = $this->getSettings();

        $key_file = PM_IDEAL_KEYS_DIR.$settings['PRIV_KEY'];
        $cert_file = PM_IDEAL_KEYS_DIR.$settings['PRIV_CERT'];

        if(!is_file($key_file) or !is_file($cert_file))
        {
            return false;
        };

        return $this->checkPrivateKeyAndCert($key_file, $cert_file, $settings['PRIV_PASS']);
    }

    function generateKeys($priv_passwd)
    {
        $return = array(
            'error' => 0
           ,'key' => 'sets_priv_key'
           ,'cert' => 'sets_priv_cert'
        );

        $dn = array(
            "countryName" => modApiFunc('Location','getCountryCode',modApiFunc('Configuration','getValue',SYSCONFIG_STORE_OWNER_COUNTRY)),
            "stateOrProvinceName" => modApiFunc('Location','getState',modApiFunc('Configuration','getValue',SYSCONFIG_STORE_OWNER_STATE)),
            "localityName" => modApiFunc('Configuration','getValue',SYSCONFIG_STORE_OWNER_CITY),
            "organizationName" => modApiFunc('Configuration','getValue',SYSCONFIG_STORE_OWNER_NAME),
            "organizationalUnitName" => modApiFunc('Configuration','getValue',SYSCONFIG_STORE_OWNER_WEBSITE),
            "commonName" => modApiFunc('Configuration','getValue',SYSCONFIG_STORE_OWNER_NAME),
            "emailAddress" => modApiFunc('Configuration','getValue',SYSCONFIG_STORE_OWNER_EMAIL),
        );

        $pkey_conf = array(
            'digest_alg' => 'sha1'
           ,'private_key_bits' => 1024
           ,'private_key_type' => OPENSSL_KEYTYPE_RSA
           ,'encrypt_key' => true
        );

        $privkey = openssl_pkey_new($pkey_conf);
        if($privkey === false)
        {
            $return['error'] = openssl_error_string();
            return $return;
        };

        $csr = openssl_csr_new($dn, $privkey);
        if($csr === false)
        {
            $return['error'] = openssl_error_string();
            return $return;
        };

        $sscert = openssl_csr_sign($csr, null, $privkey, 365);
        if($sscert === false)
        {
            $return['error'] = openssl_error_string();
            return $return;
        }

        if(!openssl_pkey_export_to_file($privkey, PM_IDEAL_KEYS_DIR.'sets_priv_key', $priv_passwd))
        {
            $return['error'] = openssl_error_string();
            return $return;
        };

        if(!openssl_x509_export_to_file($sscert, PM_IDEAL_KEYS_DIR.'sets_priv_cert'))
        {
            $return['error'] = openssl_error_string();
            return $return;
        };
        return $return;
    }

    function setCurrentIssuerID($issuer_id)
    {
        modApiFunc('Session','set','ideal_current_issuer_id',$issuer_id);
    }

    function getCurrentIssuerID()
    {
        if(modApiFunc('Session','is_set','ideal_current_issuer_id'))
        {
            return modApiFunc('Session','get','ideal_current_issuer_id');
        }
        else
        {
            return null;
        };
    }

    function getCurrentIssuerName()
    {
        $issuer_id = $this->getCurrentIssuerID();
        if($issuer_id == null)
        {
            return null;
        };

        global $application;
        $tables = $this->getTables();
        $cache_table = $tables['pm_ideal_issuers_cache']['columns'];

        $query = new DB_Select();
        $query->addSelectField($cache_table['issuer_name'], 'issuer_name');
        $query->WhereValue($cache_table['issuer_id'], DB_EQ, $issuer_id);

        $res = $application->db->getDB_Result($query);

        return count($res) == 1 ? $res[0]['issuer_name'] : null;
    }

    function getIssuersList()
    {
        $settings = $this->getSettings();
        return $settings['IL_EXP'] > time() ? $this->__getIssuersFromCache() : $this->__getIssuersFromServer();
    }

    function __getIssuersFromCache()
    {
        global $application;
        $tables = $this->getTables();
        $cache_table = $tables['pm_ideal_issuers_cache']['columns'];

        $query = new DB_Select();
        $query->addSelectTable('pm_ideal_issuers_cache');
        $query->addSelectField('*');
        $query->SelectOrder($cache_table['issuer_id'],'ASC');

        $res = $application->db->getDB_Result($query);

        return $res;
    }

    function __saveIssuersToCache($issuers)
    {
        global $application;
        $tables = $this->getTables();
        $cache_table = $tables['pm_ideal_issuers_cache']['columns'];

        $query = new DB_Delete('pm_ideal_issuers_cache');
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        loadCoreFile('db_multiple_insert.php');
        $query = new DB_Multiple_Insert('pm_ideal_issuers_cache');
        $query->setInsertFields(array('issuer_id','issuer_name'));
        foreach($issuers as $issuer_info)
        {
            $query->addInsertValuesArray(array(
                'issuer_id' => $issuer_info['issuer_id']
               ,'issuer_name' => $issuer_info['issuer_name']
            ));
        };
        $application->db->PrepareSQL($query);
        $application->db->DB_Exec();

        $sets = array('IL_EXP' => time() + PM_IDEAL_ISSUERS_LIST_EXPIRE_PERIOD);
        $this->updateSettings($sets);

        return;
    }

    function __getIssuersFromServer()
    {
        $issuers = $this->__processDirectoryRequest();
        if(!empty($issuers))
        {
            $this->__saveIssuersToCache($issuers);
        };
        return $issuers;
    }

    function __processDirectoryRequest()
    {
        $settings = $this->getSettings();

        $_timestamp = gmdate('Y-m-d').'T'.gmdate('H:i:s').'.000Z';
        $_token = '';
        $_token_code = '';

        if($settings['AUTH_TYPE'] == 'SHA1_RSA')
        {
            $msg = $_timestamp.$settings['MERCHANT_ID'].$settings['SUB_ID'];
            $msg = str_replace(array(" ","\n","\t"),"",$msg);
            $_token = $this->__createCertFingerprint(PM_IDEAL_KEYS_DIR.$settings['PRIV_CERT']);
            $_token_code = base64_encode($this->__signMessage(PM_IDEAL_KEYS_DIR.$settings['PRIV_KEY'], $settings['PRIV_PASS'], $msg));
        };

        $request_body = str_replace(
            array(
                '%TIMESTAMP%'
               ,'%MERCHANT_ID%'
               ,'%SUB_ID%'
               ,'%AUTHENTICATION%'
               ,'%TOKEN%'
               ,'%TOKEN_CODE%'
            )
           ,array(
                utf8_encode($_timestamp)
               ,utf8_encode($settings['MERCHANT_ID'])
               ,utf8_encode($settings['SUB_ID'])
               ,utf8_encode($settings['AUTH_TYPE'])
               ,utf8_encode($_token)
               ,utf8_encode($_token_code)
           )
          ,file_get_contents(PM_IDEAL_XML_TPLS_DIR.'dir_request.xml')
        );

        loadCoreFile('bouncer.php');

        $service_url = $settings['TRNS_MODE'] == PM_IDEAL_TRNS_MODE_PRODUCTION ? PM_IDEAL_LIVE_SERVICE_URL : PM_IDEAL_TEST_SERVICE_URL;

        $bnc = new Bouncer();
        $bnc->setMethod('POST');
        $bnc->setURL($service_url);
        $bnc->setHTTPversion('1.0');;
        $bnc->setPOSTtype('text/xml; charset=utf-8');
        $bnc->setPOSTstring($request_body);

        $res = $bnc->RunRequest();

        if($res === false)
        {
            return array();
        };

        $xml_response = $res['body'];
        loadCoreFile('obj_xml.php');
        $parser = new xml_doc($xml_response);
        $parser->parse();

        $issuers = array();

        foreach($parser->xml_index[0]->children as $dr_child)
        {
            if($dr_child->name == 'DIRECTORY')
            {
                foreach($dr_child->children as $d_child)
                {
                    if($d_child->name == 'ISSUER')
                    {
                        $_issuer = array('issuer_id' => '', 'issuer_name' => '');
                        foreach($d_child->children as $i_child)
                        {
                            switch($i_child->name)
                            {
                                case 'ISSUERID' :  $_issuer['issuer_id'] = $i_child->contents;   break;
                                case 'ISSUERNAME': $_issuer['issuer_name'] = $i_child->contents; break;
                            };
                        };
                        $issuers[] = $_issuer;
                    };
                };
            };
        };

        return $issuers;
    }

    function __processTransactionRequest($order_id)
    {
        $request = new Request();
        $request->setView('CheckoutView');
        //$request->setKey('asc_oid', $order_id);
        $request->setAction('UpdatePaymentStatus');
        $return_url = htmlspecialchars($request->getURL("", true));

        $currency_id = modApiFunc("Localization", "whichCurrencySendOrderToPaymentShippingGatewayIn", $order_id, $this->getUid());
        $amount = $this->export_PRICE_N_A(modApiFunc("Checkout", "getOrderPrice", "Total", $currency_id));
        $local_currency = modApiFunc("Localization", "getCurrencyCodeById", $currency_id);
        if($local_currency != 'EUR')
        {
            if(!modApiFunc('Currency_Converter','isConvertAvail',$local_currency,'EUR'))
            {
                return false;
            }
            else
            {
                $amount = modApiFunc('Currency_Converter','convert',$amount,$local_currency,'EUR');
            };
        };

        $amount = round($amount * 100);

        $issuer_id = sprintf("%04d", $this->getCurrentIssuerID());

        $settings = $this->getSettings();

        $_timestamp = gmdate('Y-m-d').'T'.gmdate('H:i:s').'.000Z';
        $_token = '';
        $_token_code = '';

        if($settings['AUTH_TYPE'] == 'SHA1_RSA')
        {
            $msg = $_timestamp .
                   $issuer_id .
                   $settings['MERCHANT_ID'] .
                   $settings['SUB_ID'] .
                   $return_url .
                   $order_id .
                   $amount . 'EUR' .
                   $settings['LANG'] .
                   $order_id . $order_id;

            //$msg = str_replace(array(" ","\n","\t"),"",$msg);
            $_token = $this->__createCertFingerprint(PM_IDEAL_KEYS_DIR.$settings['PRIV_CERT']);
            $_token_code = base64_encode($this->__signMessage(PM_IDEAL_KEYS_DIR.$settings['PRIV_KEY'], $settings['PRIV_PASS'], $msg));
        };

        $request_body = str_replace(
            array(
                '%TIMESTAMP%'
               ,'%ISSUER_ID%'
               ,'%MERCHANT_ID%'
               ,'%SUB_ID%'
               ,'%AUTHENTICATION%'
               ,'%TOKEN%'
               ,'%TOKEN_CODE%'
               ,'%RETURN_URL%'
               ,'%PURCHASE_ID%'
               ,'%AMOUNT%'
               ,'%CURRENCY%'
               ,'%EXP_PERIOD%'
               ,'%LANG%'
               ,'%DESCRIPTION%'
               ,'%ENT_CODE%'
            )
           ,array(
                utf8_encode($_timestamp)
               ,utf8_encode($issuer_id)
               ,utf8_encode($settings['MERCHANT_ID'])
               ,utf8_encode($settings['SUB_ID'])
               ,utf8_encode($settings['AUTH_TYPE'])
               ,utf8_encode($_token)
               ,utf8_encode($_token_code)
               ,utf8_encode($return_url)
               ,utf8_encode($order_id)
               ,utf8_encode($amount)
               ,utf8_encode('EUR')
               ,utf8_encode('PT1H')
               ,utf8_encode($settings['LANG'])
               ,utf8_encode($order_id)
               ,utf8_encode($order_id)
           )
          ,file_get_contents(PM_IDEAL_XML_TPLS_DIR.'trx_request.xml')
        );

        loadCoreFile('bouncer.php');

        $service_url = $settings['TRNS_MODE'] == PM_IDEAL_TRNS_MODE_PRODUCTION ? PM_IDEAL_LIVE_SERVICE_URL : PM_IDEAL_TEST_SERVICE_URL;

        $bnc = new Bouncer();
        $bnc->setMethod('POST');
        $bnc->setURL($service_url);
        $bnc->setHTTPversion('1.0');;
        $bnc->setPOSTtype('text/xml; charset=utf-8');
        $bnc->setPOSTstring($request_body);

        $res = $bnc->RunRequest();

        if($res === false)
        {
            return null;
        };

        $xml_response = $res['body'];
        loadCoreFile('obj_xml.php');
        $parser = new xml_doc($xml_response);
        $parser->parse();

        $return_data = array(
            'url' => ''
           ,'trx_id' => ''
        );

        foreach($parser->xml_index[0]->children as $tr_child)
        {
            switch($tr_child->name)
            {
                case 'ISSUER':
                    foreach($tr_child->children as $i_child)
                    {
                        if($i_child->name == 'ISSUERAUTHENTICATIONURL')
                        {
                            $return_data['url'] = $i_child->contents;
                        };
                    };
                    break;
                case 'TRANSACTION':
                    foreach($tr_child->children as $t_child)
                    {
                        if($t_child->name == 'TRANSACTIONID')
                        {
                            $return_data['trx_id'] = $t_child->contents;
                        };
                    };
                    break;
            };
        };

        return $return_data;
    }

    function __processStatusRequest($trx_id)
    {
        $settings = $this->getSettings();

        $_timestamp = gmdate('Y-m-d').'T'.gmdate('H:i:s').'.000Z';
        $_token = '';
        $_token_code = '';

        if($settings['AUTH_TYPE'] == 'SHA1_RSA')
        {
            $msg = $_timestamp .
                   $settings['MERCHANT_ID'] .
                   $settings['SUB_ID'] .
                   $trx_id;

            $_token = $this->__createCertFingerprint(PM_IDEAL_KEYS_DIR.$settings['PRIV_CERT']);
            $_token_code = base64_encode($this->__signMessage(PM_IDEAL_KEYS_DIR.$settings['PRIV_KEY'], $settings['PRIV_PASS'], $msg));
        };

        $request_body = str_replace(
            array(
                '%TIMESTAMP%'
               ,'%MERCHANT_ID%'
               ,'%SUB_ID%'
               ,'%AUTHENTICATION%'
               ,'%TOKEN%'
               ,'%TOKEN_CODE%'
               ,'%TRX_ID%'
            )
           ,array(
                utf8_encode($_timestamp)
               ,utf8_encode($settings['MERCHANT_ID'])
               ,utf8_encode($settings['SUB_ID'])
               ,utf8_encode($settings['AUTH_TYPE'])
               ,utf8_encode($_token)
               ,utf8_encode($_token_code)
               ,utf8_encode($trx_id)
           )
          ,file_get_contents(PM_IDEAL_XML_TPLS_DIR.'stat_request.xml')
        );

        loadCoreFile('bouncer.php');

        $service_url = $settings['TRNS_MODE'] == PM_IDEAL_TRNS_MODE_PRODUCTION ? PM_IDEAL_LIVE_SERVICE_URL : PM_IDEAL_TEST_SERVICE_URL;

        $bnc = new Bouncer();
        $bnc->setMethod('POST');
        $bnc->setURL($service_url);
        $bnc->setHTTPversion('1.0');;
        $bnc->setPOSTtype('text/xml; charset=utf-8');
        $bnc->setPOSTstring($request_body);

        $res = $bnc->RunRequest();

        if($res === false)
        {
            return null;
        };

        $xml_response = $res['body'];
        loadCoreFile('obj_xml.php');
        $parser = new xml_doc($xml_response);
        $parser->parse();

        $status = null;

        foreach($parser->xml_index[0]->children as $sr_child)
        {
            if($sr_child->name == 'TRANSACTION')
            {
                foreach($sr_child->children as $t_child)
                {
                    if($t_child->name == 'STATUS')
                    {
                        $status = $t_child->contents;
                    };
                };
            };
        };

        return $status;
    }

    function __createCertFingerprint($cert_path)
    {
        $data = openssl_x509_read(file_get_contents($cert_path));
        openssl_x509_export($data, $data);
        $data = str_replace(array('-----BEGIN CERTIFICATE-----','-----END CERTIFICATE-----'),'',$data);
        $data = base64_decode($data);
        $fingerprint = _ml_strtoupper(sha1($data));
        return $fingerprint;
    }

    function __signMessage($key_path, $key_pass, $data)
    {
        $data = preg_replace("/\s/","",$data);
        $pkey = openssl_pkey_get_private(file_get_contents($key_path), $key_pass);
        openssl_sign($data, $signature, $pkey);
        openssl_free_key($pkey);
        return $signature;
    }

    function getBodyOnLoad()
    {
        $script = "window.location = '" . $this->trx_url . "';";
        return $script;
    }


// local data
    /**
     * used for transmitting url to form
     *
     * @var string
     */
    var $trx_url = "";
};

?>