<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Shipping Module "Federal Express".
 *
 * @package ShippingModuleFedEx
 * @author Egor V. Derevyankin
 */

define("FDX_DEFAULT_RESIDENTAL_ADDON", 2.10);
define("FDX_HOME_RESIDENTAL_ADDON", 1.75);
define("FDX_NON_RESIDENTAL_ADDON", 1.30);

define("FDX_DAS_RESIDENTAL_ADDON", 2.10);
define("FDX_DAS_NON_RESIDENTAL_ADDON", 1.30);
define("FDX_DAS_ALASKA_EXPRESS_ADDON", 15);
define("FDX_DAS_ALASKA_GROUND_ADDON", 30);
define("FDX_DAS_ALASKA_BIG_GROUND_ADDON", 100);
define("FDX_DAS_HAWAII_GROUND_ADDON", 11);

define("FDX_NORTH_CANADA_ADDON", 65);

class Shipping_Module_FedEx extends pm_sm_api
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /**
     * ShippingModuleFedEx  constructor.
     */
    function Shipping_Module_FedEx()
    {
        $this->is_debug=false;
        $this->_debug_info=array();
    }

    /* static */ function getInitialCurrencySettings()
    {
        $res = array
        (
            'currency_acceptance_rules' => array
            (
                array
                (
                    'rule_name'     =>  ACTIVE_AND_ACCEPTED_BY_PM_SM_AND_SELECTED_BY_CUSTOMER
                   ,'rule_selected' =>  DB_TRUE
                )
            )

           ,'accepted_currencies' => array
            (
                array
                (
                    'currency_code'   => 'CAD'
                   ,'currency_status' => 'ACCEPTED'
                )
               ,array
                (
                    'currency_code'   => 'USD'
                   ,'currency_status' => 'ACCEPTED'
                )
            )
        );
        return $res;
    }
	function getUID()
	{
	    // FedEx GUID
		return "A15365CD-D821-4170-B489-584DFFAC5A9C";
	}

    /**
     * @return Info about this shipping module
     */
    function getInfo()
    {

        global $application;
        $obj = &$application->getInstance('MessageResources',"shipping-module-fedex-messages", "AdminZone");

        $this->getSettings();

        return array("GlobalUniqueShippingModuleID" => $this->getUID(),
                     "Name"        => $this->Settings["MODULE_NAME"],
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$obj->getMessage('MODULE_STATUS_ACTIVE')."</span>":$obj->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>", //NOT for active/inactive.
                     "PreferencesAZViewClassName" => "fedex_input_az",
                     "APIClassName" => __CLASS__
                    );
    }

    /**
     * Check requirments.
     *
     * @return array with the failed requirments
     */
    function checkRequirments()
    {
        return array();
    }

    /**
     * Module installer.
     */
    function install()
    {
        _use(dirname(__FILE__).'/includes/install.inc');

        //                                RemovePersonInfoType
        //                   ,          -     CheckoutEditor
        //             PersonInfoType,         , BillingInfo, ShippingInfo,
        //  CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
        //             ), ShippingMethodInfo (              ).
        //       ShippingModule'                                                 ,
        //                                     .
        //                    ,                 PersonInfoType          .
        //
        //  PersonInfoType.
        modApiFunc('EventsManager',
               'addEventHandler',
               'RemovePersonInfoTypeEvent',
               'Shipping_Module_FedEx',
               'OnRemovePersonInfoType');
    }

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
     *            ), ShippingMethodInfo (              ).
     *      ShippingModule'                                                 ,
     *                                    .
     *                   ,                 PersonInfoType          .
     *
     * PersonInfoType.
     *
     *        FedEx                         PersonInfoType:
     * shippingInfo
     *
     * @param string $person_info_type_tag - tag      person info type' ,
     *                      (turn off)   CheckoutFormEditor' .
     *
     * @return mixed - NULL,                       - "          ",       -
     *                                     ,            ,
     *        (                                   ).
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if($this->isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this->MessageResources = &$application->getInstance('MessageResources',"shipping-module-fedex-messages", "AdminZone");
            $msg = $this->MessageResources->getMessage('MODULE_SHIPPING_FEDEX_API_MSG_REMOVE_PERSON_INFO_TYPE');

            $value = NULL;
            switch($person_info_type_tag)
            {
                case "shippingInfo":
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }


    /**
     * @return true if module is active or false if is not
     */
    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules", "shipping");
        return array_key_exists($this->getUid(), $active_modules);
    }

    /**
     * Uninstalls the module.
     */
    function uninstall()
    {
        $query = new DB_Table_Delete(Shipping_Module_FedEx::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    /**
     * @param $method_id ID of shipping method of this module
     * @return true if ID is valid or false if it is not
     */
    function isValidShippingMethodId($method_id)
    {
        $check_result = execQuery('SELECT_SM_FEDEX_METHODS_BY_METHOD_ID',array('ApiClassName' => __CLASS__, 'method_id' => $method_id));

        if(isset($check_result[0]['id']))
            return true;
        else
            return false;
    }

    /**
     * @param $method_id ID of shipping method of this module
     * @param $and_calc true if it is necessary to calculate the shipping cost
     *         or false if it is not
     * @return array with the shipping method info (id,method_name,method_code,
     *         destination,available,cost)
     */
    function getShippingMethodInfo($method_id,$and_calc=false)
    {
        global $application;
        $tables=$this->getTables();
        $methods_table=$tables['sm_fedex_methods']['columns'];

        $select_result = execQuery('SELECT_SM_FEDEX_METHODS_BY_METHOD_ID_EXT',array('ApiClassName' => __CLASS__, 'method_id' => $method_id));

        $method=$select_result[0];
        $method["cost"]=PRICE_N_A;

        if($and_calc!=false)
        {
            //$ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
            //$pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
            $ShippingInfo = modApiFunc("Shipping_Cost_Calculator","getShippingInfo");
            $pak_weight = modApiFunc("Shipping_Cost_Calculator","getPackWeight");
            $rate=$this->getRateFromCache($ShippingInfo,$pak_weight,$method_id);
            if($rate!=false)
                $method["cost"]=$rate;
        };

        return $method;
    }

    /**
     * @return structure of the tables of this module
     */
    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array ();

        $settings = 'sm_fedex_settings';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                => $settings.'.sm_fedex_setting_id'
               ,'key'               => $settings.'.sm_fedex_setting_key'
               ,'value'             => $settings.'.sm_fedex_setting_value'
            );
        $tables[$settings]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'key'               => DBQUERY_FIELD_TYPE_CHAR50
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        $methods = 'sm_fedex_methods';
        $tables[$methods] = array();
        $tables[$methods]['columns'] = array
            (
                'id'                => $methods.'.sm_fedex_method_id'
               ,'method_code'       => $methods.'.sm_fedex_method_code'
               ,'method_name'       => $methods.'.sm_fedex_method_name'
               ,'destination'       => $methods.'.sm_fedex_method_destination'
               ,'available'         => $methods.'.sm_fedex_method_available'
            );
        $tables[$methods]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'method_code'       => DBQUERY_FIELD_TYPE_CHAR50
               ,'method_name'       => DBQUERY_FIELD_TYPE_CHAR255
               ,'destination'       => DBQUERY_FIELD_TYPE_CHAR1 .' default \'L\''
               ,'available'         => DBQUERY_FIELD_TYPE_CHAR1 .' default \'N\''
            );
        $tables[$methods]['primary'] = array
            (
                'id'
            );

        $rates_cache = 'sm_fedex_rates_cache';
        $tables[$rates_cache] = array();
        $tables[$rates_cache]['columns'] = array
            (
                'id'                => $rates_cache.'.sm_fedex_crate_id'
               ,'hash'              => $rates_cache.'.sm_fedex_crate_hash'
               ,'method_id'         => $rates_cache.'.sm_fedex_crate_method_id'
               ,'rate'              => $rates_cache.'.sm_fedex_crate_rate'
               ,'expire'            => $rates_cache.'.sm_fedex_crate_expire'
            );
        $tables[$rates_cache]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'hash'              => DBQUERY_FIELD_TYPE_CHAR50
               ,'method_id'         => DBQUERY_FIELD_TYPE_INT
               ,'rate'              => DBQUERY_FIELD_TYPE_DECIMAL20_5
               ,'expire'            => DBQUERY_FIELD_TYPE_INT
            );
        $tables[$rates_cache]['primary'] = array
            (
                'id'
            );

        global $application;
        return $application->addTablePrefix($tables);
    }


    /**
     *
     */
    function clearSettingsInDB()
    {

        global $application;
        $query = new DB_Delete('sm_fedex_settings');
        $application->db->getDB_Result($query);

    }

    /**
     * @return array of the settings of this module
     */
    function getSettings()
    {
        $result = execQuery('SELECT_PM_SM_SETTINGS',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'sm_fedex_settings'));
        $this->Settings = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
            $this->Settings[$result[$i]['set_key']] = unserialize($result[$i]['set_value']);
        }
        return $this->Settings;
    }

    /**
     * Sets settings for this module.
     *
     * @param $Settings array of the all settings of this module
     */
    function setSettings($Settings)
    {
        global $application;
        $this->clearSettingsInDB();
        $tables = $this->getTables();
        $columns = $tables['sm_fedex_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Insert('sm_fedex_settings');
            $query->addInsertValue($key, $columns['key']);
            $query->addInsertValue(serialize($value), $columns['value']);
            $application->db->getDB_Result($query);

            $inserted_id = $application->db->DB_Insert_Id();
        }
    }

    /**
     * Updates settings for this module.
     *
     * @param $Settings array of the updated settings of this module
     */
    function updateSettings($Settings)
    {
        global $application;
        $tables = $this->getTables();
        $columns = $tables['sm_fedex_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Update('sm_fedex_settings');
            $query->addUpdateValue($columns['value'], serialize($value));
            $query->WhereValue($columns['key'], DB_EQ, $key);
            $application->db->getDB_Result($query);
        }
    }

    /**
     * @param $type type of the requested methods (ALL|AVAILABLE|NOT_AVAILABLE)
     * @param $and_calc true if it is necessary to calculate the shipping cost
     *        or false if it is not
     * @return array of the shipping methods arrays, every will be as
     *        (id,method_name,method_code,destination,available,cost)
     */
    function getShippingMethods($type="ALL",$and_calc=false)
    {
        $available_condition=($type=="AVAILABLE"?"'Y'":($type=="NOT_AVAILABLE"?"'N'":""));
        $methods = execQuery('SELECT_SM_FEDEX_METHODS_BY_AVAILABILITY',array('ApiClassName' => __CLASS__, 'available_condition' => $available_condition));

        if($and_calc!=false and !empty($methods))
        {
            /*
            if($this->is_debug==true)
            {
                $ShippingInfo = modApiFunc("Shipping_Tester","getShippingInfo");
                $pak_weight=modApiFunc("Shipping_Tester","getPackWeight");
            }
            else
            {
                $ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
                $pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
            }
            */
            $ShippingInfo = modApiFunc("Shipping_Cost_Calculator","getShippingInfo");
            $pak_weight=modApiFunc("Shipping_Cost_Calculator","getPackWeight");

            $_rates=array();
            $cmethods=array();


            if($ShippingInfo["isMet"]!=false)
            {
                $_rates=$this->calculate_rates($ShippingInfo,$pak_weight);
            };

            if(!empty($_rates))
            {
                foreach($_rates as $service => $sdata)
                {
                    foreach($methods as $key => $method)
                    {
                            if($method['method_code']==$service)
                                $cmethods[]=array_merge($method,$sdata);
                    };
                };
            };

            $methods=$cmethods;

            $this->saveRatesToCache($ShippingInfo,$pak_weight,$methods);
        };

        return $methods;
    }

    /**
     * @param $update_array array if the IDs of the available methods,
     *      all other methods will be set to 'not available'
     */
    function updateShippingMethods($update_array)
    {
        global $application;

        $tables=$this->getTables();
        $methods_table=$tables['sm_fedex_methods']['columns'];

        $query = new DB_Update('sm_fedex_methods');
        $query->addUpdateValue($methods_table['available'],"N");
        $application->db->getDB_Result($query);

        if(count($update_array)==0 or !is_array($update_array))
            return;

        $query = new DB_Update('sm_fedex_methods');
        $query->addUpdateValue($methods_table['available'],"Y");
        $query->Where($methods_table['id'], DB_IN, "('".implode("','",array_keys($update_array))."')");
        $application->db->getDB_Result($query);

        return;
    }

    /**
     * @param $s_info shipping info of the destination
     * @param $weight weight of the package
     * @param $rates array of the shipping rates for saving
     */
    function saveRatesToCache($s_info,$weight,$rates)
    {
        if($s_info["isMet"]==false or empty($rates))
            return;

        $s_country=modApiFunc("Location","getCountry",$s_info["validatedData"]["Country"]["value"]);
        $s_zipcode=$s_info["validatedData"]["Postcode"]["value"];
        $s_name = isset($s_info["validatedData"]["Firstname"]["value"]) ? $s_info["validatedData"]["Firstname"]["value"] : '';
        $s_name.= isset($s_info["validatedData"]["Lastname"]["value"])  ? $s_info["validatedData"]["Lastname"]["value"]  : '';

        $hash=md5($s_country.$s_name.$s_zipcode.$weight);

        $tables=$this->getTables();
        $rc_table=$tables['sm_fedex_rates_cache']['columns'];

        global $application;

        $query = new DB_Delete('sm_fedex_rates_cache');
        $query->Where($rc_table['hash'], DB_EQ, "'$hash'");
        $application->db->getDB_Result($query);

        $query = new DB_Delete('sm_fedex_rates_cache');
        $query->Where($rc_table['expire'], DB_LT, time());
        $application->db->getDB_Result($query);

        foreach($rates as $key => $rate_info)
        {
            $query = new DB_Insert('sm_fedex_rates_cache');
            $query->addInsertValue($hash, $rc_table["hash"]);
            $query->addInsertValue($rate_info["id"],$rc_table["method_id"]);
            $query->addInsertValue($rate_info["cost"],$rc_table["rate"]);
            $query->addInsertValue(time()+18000,$rc_table["expire"]);
            $application->db->getDB_Result($query);
        };

        return;
    }

    /**
     * @param $s_info shipping info of the destination
     * @param $weight weight of the package
     * @param $method_id ID of the required shipping method
     * @return cost of the shipping from the cache
     *      or false if the requested cost is not saved in the cache
     */
    function getRateFromCache($s_info,$weight,$method_id)
    {
        if($s_info["isMet"]==false or !is_numeric($method_id))
            return false;

        $s_country=modApiFunc("Location","getCountry",$s_info["validatedData"]["Country"]["value"]);
        $s_zipcode=$s_info["validatedData"]["Postcode"]["value"];
        $s_name = isset($s_info["validatedData"]["Firstname"]["value"]) ? $s_info["validatedData"]["Firstname"]["value"] : '';
        $s_name.= isset($s_info["validatedData"]["Lastname"]["value"])  ? $s_info["validatedData"]["Lastname"]["value"]  : '';

        $hash=md5($s_country.$s_name.$s_zipcode.$weight);

        $tables=$this->getTables();
        $rc_table=$tables['sm_fedex_rates_cache']['columns'];

        global $application;

        $cached = execQuery('SELECT_SM_FEDEX_CACHED_RATES',array('ApiClassName' => __CLASS__, 'hash' => $hash, 'method_id' => $method_id));

        if(isset($cached[0]["rate"]))
            return $cached[0]["rate"];
        else
            return false;
    }

    /**
     * @param $ShippingInfo shipping info of the destination
     * @param $pak_weight weight of the package
     * @return array of the shipping rates arrays
     */
    function calculate_rates($ShippingInfo,$pak_weight)
    {
        return $this->CalculateRatesByRealTime($ShippingInfo,$pak_weight);
    }

    /**
     * Calculates the shipping rates by the Real-Time method.
     *
     * @param $ShippingInfo shipping info of the destination
     * @param $pak_weight weight of the package
     * @return array of the shipping rates arrays
     */
    function CalculateRatesByRealTime($ShippingInfo,$pak_weight)
    {
        global $application;
        $this->MessageResources = &$application->getInstance('MessageResources',"shipping-module-fedex-messages", "AdminZone");
        if($this->is_debug == true)
        {
            $this->_debug_info[]=array("Calculate Type", "Real-Time");
        };

        # get settings
        $Settings = $this->getSettings();
        $_rates = array();

        # calculate the midnight of today
        $today = getdate();
        $today_midnight = mktime(0,0,0,$today['mon'],$today['mday'],$today['year']);

        # countries which need state
        $need_state = array("CA","US");

        # get origin and destination zip and country
        $ozip = modApiFunc("Configuration","getTagValue","StoreOwnerPostcode");
        $dzip = $ShippingInfo["validatedData"]["Postcode"]["value"];
        $ocntr = modApiFunc("Location","getCountryCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_COUNTRY));
        $oStreetLines = modApiFunc("Configuration","getTagValue","StoreOwnerStreetLine1");
        $oStreetLines .= ' '.modApiFunc("Configuration","getTagValue","StoreOwnerStreetLine2");
        $oCity = modApiFunc("Configuration","getTagValue","StoreOwnerCity");
        $dcntr = modApiFunc("Location","getCountryCode",$ShippingInfo["validatedData"]["Country"]["value"]);
        $dStreetLines = $ShippingInfo["validatedData"]["Streetline1"]["value"];
        $dStreetLines .= ' '.$ShippingInfo["validatedData"]["Streetline2"]["value"];
        $dCity = $ShippingInfo["validatedData"]["City"]["value"];

        # need state ?
        $ostate = $dstate = '';
        if(in_array($ocntr,$need_state))
        {
            $ostate = modApiFunc("Location","getStateCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_STATE));
        };
        if(in_array($dcntr,$need_state))
        {
            $dstate = modApiFunc("Location","getStateCode",$ShippingInfo["validatedData"]["Statemenu"]["value"]);
        };

        # Get the declared value of package
        $decl_value = modApiFunc("Shipping_Cost_Calculator","getCartSubtotal");
        $main_currency_id = modApiFunc("Localization", "getMainStoreCurrency");
        $local_currency_code = modApiFunc("Localization", "getCurrencyCodeById", $main_currency_id);

        if($ocntr == "CA" and $dcntr == "CA")
        {
            $need_currency_code = "CAD";
        }
        else
        {
            $need_currency_code = "USD";
        };

        if($local_currency_code != $need_currency_code)
        {
            if(modApiFunc('Currency_Converter','isConvertAvail',$local_currency_code,$need_currency_code))
            {
                $new_decl_value = modApiFunc('Currency_Converter','convert',$decl_value,$local_currency_code,$need_currency_code);

                if($this->is_debug == true)
                {
                    $msg = $this->MessageResources->getMessage('LABEL_CONVERTING');
                    $this->_debug_info[] = array($msg, "{$local_currency_code} -> {$need_currency_code}: {$decl_value} -> {$new_decl_value}");
                };

                $decl_value = $new_decl_value;
            }
            else
            {
                //                                 ->                  .
                if($this->is_debug == true)
                {
                    $msg = $this->MessageResources->getMessage('ERROR_014');
                    $this->_debug_info[] = array('Error', $msg);
                };

                return $_rates;
            };
        };

        # Calculate weight
        $local_wcoeff = modApiFunc("Localization","getValue","WEIGHT_COEFF");
        $weight_in_lbs = round(($pak_weight * $local_wcoeff * 1000 / 453.6),1);

        if($weight_in_lbs < 1)
        {
            $weight_in_lbs = 1;
        };

        # Calculate dimension sizes
        $dimensions_defined = false;
        if($Settings["OPTS_PACKAGING"] == "YOURPACKAGING")
        {
            if($Settings["OPTS_MPK_H"] != "" and $Settings["OPTS_MPK_W"] != "" and $Settings["OPTS_MPK_L"] != "")
            {
                $conv2in = ($Settings["OPTS_MPK_UNITS"] == "IN" ? 1 : 0.39);
                $pak_height_in = round($Settings["OPTS_MPK_H"] * $conv2in);
                $pak_width_in = round($Settings["OPTS_MPK_W"] * $conv2in);
                $pak_length_in = round($Settings["OPTS_MPK_L"] * $conv2in);
                $dimensions_defined=true;
            };
        };

        if($this->is_debug == true)
        {
            $this->_debug_info[] = array("Destination Country Code", modApiFunc("Location","getCountryCode",$ShippingInfo["validatedData"]["Country"]["value"]));
            $this->_debug_info[] = array("Destination Zip", $dzip);
            $this->_debug_info[] = array("Residental delivery", $Settings["OPTS_SHIP2RES"]);
            $this->_debug_info[] = array("Service URL is", "https://".$Settings["SRVC_URL"]);
        };

        # Start rate calculation

        $ship2res = ($Settings["OPTS_SHIP2RES"] == "YES" ? true : false);

        # define needed methods - Express, Ground or both
        $CARRIERS = array();
        if($Settings["OPTS_CARRIER"] == "EXP" or $Settings["OPTS_CARRIER"] == "BOTH")
        {
            $CARRIERS[] = "FDXE";
        };
        if($Settings["OPTS_CARRIER"] == "GND" or $Settings["OPTS_CARRIER"] == "BOTH")
        {
            $CARRIERS[] = "FDXG";
        };

        # create the Bouncer object for sending the request
        loadCoreFile('bouncer.php');
        $bnc = new Bouncer();
        $bnc->setMethod("POST");
        $bnc->setURL('https://'.$Settings["SRVC_URL"]);

        foreach($CARRIERS as $carrier)
        {
            # generate XML-request
            $xml_request = ''
                .'<?xml version="1.0" encoding="UTF-8"?>'
                .'<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="http://fedex.com/ws/rate/v10">'
                    .'<SOAP-ENV:Body>'
                        .'<ns1:RateRequest>'
                            .'<ns1:WebAuthenticationDetail>'
                                .'<ns1:UserCredential>'
                                    .'<ns1:Key>'.$Settings['SRVC_AUTH_KEY'].'</ns1:Key>'
                                    .'<ns1:Password>'.$Settings['SRVC_AUTH_PASS'].'</ns1:Password>'
                                .'</ns1:UserCredential>'
                            .'</ns1:WebAuthenticationDetail>'
                            .'<ns1:ClientDetail>'
                                .'<ns1:AccountNumber>'.$Settings['SRVC_ACCOUNT'].'</ns1:AccountNumber>'
                                .'<ns1:MeterNumber>'.$Settings['SRVC_METER'].'</ns1:MeterNumber>'
                            .'</ns1:ClientDetail>'
                            .'<ns1:TransactionDetail>'
                                .'<ns1:CustomerTransactionId>'.uniqid(mt_rand(0,1000)).'</ns1:CustomerTransactionId>'
                            .'</ns1:TransactionDetail>'
                            .'<ns1:Version>'
                                .'<ns1:ServiceId>crs</ns1:ServiceId>'
                                .'<ns1:Major>10</ns1:Major>'
                                .'<ns1:Intermediate>0</ns1:Intermediate>'
                                .'<ns1:Minor>0</ns1:Minor>'
                            .'</ns1:Version>'
                            .'<ns1:ReturnTransitAndCommit>true</ns1:ReturnTransitAndCommit>'
                            .'<ns1:RequestedShipment>'
                                .'<ns1:ShipTimestamp>'.date('c').'</ns1:ShipTimestamp>'
                                .'<ns1:DropoffType>'.$Settings['OPTS_DROP_OFF'].'</ns1:DropoffType>'
                                .'<ns1:PackagingType>'.$Settings['OPTS_PACKAGING'].'</ns1:PackagingType>'
                                .'<ns1:Shipper>'
                                    .'<ns1:Address>'
                                        .'<ns1:StreetLines>'.$oStreetLines.'</ns1:StreetLines>'
                                        .'<ns1:City>'.$oCity.'</ns1:City>'
                                        .'<ns1:StateOrProvinceCode>'.$ostate.'</ns1:StateOrProvinceCode>'
                                        .'<ns1:PostalCode>'.$ozip.'</ns1:PostalCode>'
                                        .'<ns1:CountryCode>'.$ocntr.'</ns1:CountryCode>'
                                    .'</ns1:Address>'
                                .'</ns1:Shipper>'
                                .'<ns1:Recipient>'
                                    .'<ns1:Address>'
                                        .'<ns1:StreetLines>'.$dStreetLines.'</ns1:StreetLines>'
                                        .'<ns1:City>'.$dCity.'</ns1:City>'
                                        .'<ns1:StateOrProvinceCode>'.$dstate.'</ns1:StateOrProvinceCode>'
                                        .'<ns1:PostalCode>'.$dzip.'</ns1:PostalCode>'
                                        .'<ns1:CountryCode>'.$dcntr.'</ns1:CountryCode>'
                                        .($ship2res?'<ns1:Residential>true</ns1:Residential>':'')
                                    .'</ns1:Address>'
                                .'</ns1:Recipient>'
                                .'<ns1:ShippingChargesPayment>'
                                    .'<ns1:PaymentType>SENDER</ns1:PaymentType>'
                                    .'<ns1:Payor>'
                                        .'<ns1:AccountNumber>'.$Settings['SRVC_ACCOUNT'].'</ns1:AccountNumber>'
                                        .'<ns1:CountryCode>'.$ocntr.'</ns1:CountryCode>'
                                    .'</ns1:Payor>'
                                .'</ns1:ShippingChargesPayment>'
                                .'<ns1:RateRequestTypes>LIST</ns1:RateRequestTypes>'
                                .'<ns1:PackageCount>1</ns1:PackageCount>'
                                .'<ns1:RequestedPackageLineItems>'
                                    .'<ns1:SequenceNumber>1</ns1:SequenceNumber>'
                                    .'<ns1:GroupPackageCount>1</ns1:GroupPackageCount>'
                                    .'<ns1:Weight>'
                                        .'<ns1:Units>LB</ns1:Units>'
                                        .'<ns1:Value>'.$weight_in_lbs.'</ns1:Value>'
                                    .'</ns1:Weight>'
                                    .($dimensions_defined ? (
                                        '<ns1:Dimensions>'
                                            .'<ns1:Length>'.$pak_length_in.'</ns1:Length>'
                                            .'<ns1:Width>'.$pak_width_in.'</ns1:Width>'
                                            .'<ns1:Height>'.$pak_height_in.'</ns1:Height>'
                                            .'<ns1:Units>IN</ns1:Units>'
                                        .'</ns1:Dimensions>') : '')
                                .'</ns1:RequestedPackageLineItems>'
                            .'</ns1:RequestedShipment>'
                        .'</ns1:RateRequest>'
                    .'</SOAP-ENV:Body>'
                .'</SOAP-ENV:Envelope>';

        if($this->is_debug==true)
        {
            $this->_debug_info[] = array("XML Request for '$carrier'", $xml_request);
        };

         # send request to FedEx server and receive response
        $bnc->setPOSTstring($xml_request);
        $res = $bnc->RunRequest();

        if($res!=false) # if we have the response
        {
            $xml_response = $res["body"];

            if($this->is_debug == true)
            {
                $this->_debug_info[] = array("XML Response for '$carrier'", $xml_response);
            };

            $xml_response = str_replace(array('<soapenv:', '<env:', '<SOAP-ENV:', '<ns:', '<v10:'), '<', $xml_response);
            $xml_response = str_replace(array('</soapenv:', '</env:', '</SOAP-ENV:', '</ns:', '</v10:'), '</', $xml_response);

            # parse the XML-response
            loadCoreFile('obj_xml.php');
            $parser = new xml_doc($xml_response);
            $parser->parse();

            $entries = $parser->xml_index[0]->children;

            if($parser->findTag('FAULT') !==false)
            {
                $_ecode = $parser->findTag('CON:ERRORCODE')->contents;
                $_emsg = $parser->findTag('CON:REASON')->contents;
                if($this->is_debug == true)
                {
                    $this->_debug_info[] = array("Error", $_ecode.": ".$_emsg);
                }
                CTrace::err(array("Error", $_ecode.": ".$_emsg, $xml_response));
            }
            elseif($parser->findTag('RATEREPLYDETAILS') === false)
            {
                $_ecode = $parser->findTag('CODE')->contents;
                $_emsg = $parser->findTag('MESSAGE')->contents;
                if($this->is_debug == true)
                {
                    $this->_debug_info[] = array("Error", $_ecode.": ".$_emsg);
                }
                CTrace::err(array("Error", $_ecode.": ".$_emsg, $xml_response));
            }
            else
            {
            foreach($entries as $k => $entry)
            {
                if($entry->name == "BODY")
                {
                    $_service = "";   # name of the service
                    $_days = -1;      # delivery time of the service
                    $_rate = 0;       # cost of the service
                    $_currency = "";  #
                    foreach($entry->children as $ek => $echild)
                    {
                        if($echild->name == "RATEREPLY")
                        {
                            foreach($echild->children as $rrk => $rrchild)
                            {
                                if($rrchild->name == "RATEREPLYDETAILS")
                                {
                                    foreach($rrchild->children as $rrdk => $rrdchild)
                                    {
                                        if($rrdchild->name == 'SERVICETYPE')
                                        {
                                            $_service = $rrdchild->contents;
                                        }

                                        //if($rrdchild->name == 'TRANSITTIME')
                                        //    $_days = $rrdchild->contents;

                                        if($rrdchild->name == 'RATEDSHIPMENTDETAILS')
                                            foreach($rrdchild->children as $rsdk => $rsdchild)
                                                if($rsdchild->name == 'SHIPMENTRATEDETAIL')
                                                    foreach($rsdchild->children as $srdk => $srdchild)
                                                        if($srdchild->name == 'TOTALNETCHARGE')
                                                            foreach($srdchild->children as $tnck => $tncchild)
                                                            {
                                                                if($tncchild->name == 'AMOUNT')
                                                                    $_rate = $tncchild->contents;
                                                                if($tncchild->name == 'CURRENCY')
                                                                    $_currency = $tncchild->contents;

                                                                if($_currency !='' && $_currency != $local_currency_code)
                                                                {
                                                                    if(modApiFunc('Currency_Converter','isConvertAvail',$_currency,$local_currency_code))
                                                                    {
                                                                        $new_rate = modApiFunc('Currency_Converter','convert',$_rate,$_currency,$local_currency_code);
                                                                        if($this->is_debug == true)
                                                                        {
                                                                            $msg = $this->MessageResources->getMessage('LABEL_CONVERTING');
                                                                            $this->_debug_info[] = array($msg, "{$_currency} -> {$local_currency_code}: {$_rate} -> {$new_rate}");
                                                                        };
                                                                        $_rate = $new_rate;
                                                                    }
                                                                    else
                                                                    {
                                                                        break;
                                                                    };
                                                                };

                                                            }
                                        $_rates[$_service] = array("cost" => $_rate, "days" => $_days);
                                    }
                                }
                            };
                        };
                    };
                    }
                };
                }
            }
            elseif($this->is_debug == true)
            {
                $this->_debug_info[] = array("Error",$bnc->getLastErrorMessage());
            };

        };

        return $_rates;
    }

    function setDebugFlag($flag=true)
    {
        if(is_bool($flag))
            $this->is_debug=$flag;
    }

    function getDebugInfo()
    {
        return $this->_debug_info;
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * : DELETE THIS OLD FUNCTION!
     * Returns the shipping cost by shipping method ID.
     *
     * @param $method_id shipping method ID
     * @return shipping cost
     */
    function getShippingCost($method_id="")
    {
        $ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
        $pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
        $rate=$this->getRateFromCache($ShippingInfo,$pak_weight,$method_id);
        if($rate!=false)
            return $rate;
        else
            return PRICE_N_A;
    }

    var $bActive;

    # these vars uses for offline calculation
    var $_rates;
    var $_sets;

    var $is_debug;
    var $_debug_info;

    /**#@-*/
}
?>