<?php
/***********************************************************************
| Avactis (TM) Shopping Cart software developed by Pentasoft Corp.
| http://www.avactis.com
| -----------------------------------------------------------------------
| All source codes & content (c) Copyright 2004-2010, Pentasoft Corp.
| unless specifically noted otherwise.
| =============================================
| This source code is released under the Avactis License Agreement.
| The latest version of this license can be found here:
| http://www.avactis.com/license.php
|
| By using this software, you acknowledge having read this license agreement
| and agree to be bound thereby.
|
 ***********************************************************************/
?><?php

/**
 * Shipping Module "InterShipper"
 *
 * @package ShippingModuleInterShipper
 * @author Egor V. Derevyankin
 */
class Shipping_Module_InterShipper extends pm_sm_api
{
//------------------------------------------------
//               PUBLIC DECLARATION
//------------------------------------------------

    /**#@+
     * @access public
     */

    /* static */ function getInitialCurrencySettings()
    {
        $res = array
        (
            'currency_acceptance_rules' => array
            (
                array
                (
                    'rule_name'     =>  THE_ONLY_ACCEPTED
                   ,'rule_selected' =>  DB_TRUE
                )
            )

           ,'accepted_currencies' => array
            (
                array
                (
                    'currency_code'   => 'USD'
                   ,'currency_status' => 'THE_ONE_ONLY_ACCEPTED'
                )
            )
        );
        return $res;
    }

    /**
     * ShippingModuleInterShipper  constructor.
     */
    function Shipping_Module_InterShipper()
    {
    }

	function getUID()
	{
		return "F227F4AC-98B8-40D8-96EE-0E733CC58F79";
	}

    /**
     * @return Info about this shipping module
     */
    function getInfo()
    {

        global $application;
        $obj = &$application->getInstance('MessageResources',"shipping-module-intershipper-messages", "AdminZone");

        $this->getSettings();

        return array("GlobalUniqueShippingModuleID" => $this->getUID(),
                     "Name"        => $this->Settings["MODULE_NAME"],
                     "Description" => $this->Settings["MODULE_DESCR"],
                     "StatusMessage" => ($this->IsActive())? "<span class=\"status_online\">".$obj->getMessage('MODULE_STATUS_ACTIVE')."</span>":$obj->getMessage('MODULE_STATUS_INACTIVE')."<span class=\"required\">*</span>", //NOT for active/inactive.
                     "PreferencesAZViewClassName" => "intershipper_input_az",
                     "APIClassName" => __CLASS__
                    );
    }

    /**
     * Check requirments.
     *
     * @return array with the failed requirments
     */
    function checkRequirments()
    {
        $return=array();

        // check SSL
        loadCoreFile('bouncer.php');
        $bssl = new Bouncer();
        if(!$bssl->isSSLavailable())
            $return[]="SSL";

        //check origin country
        $ocntr = modApiFunc("Location","getCountryCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_COUNTRY));
        if($ocntr!="US")
            $return[]="OCNTR";

        //check local currency
        $main_currency_id = modApiFunc("Localization", "getMainStoreCurrency");
        $local_currency_code = modApiFunc("Localization", "getCurrencyCodeById", $main_currency_id);
        if($local_currency_code!="USD")
            $return[]="CURRENCY";

        return $return;
    }

    /**
     * Module installator.
     */
    function install()
    {
        _use(dirname(__FILE__).'/includes/install.inc');

        //                                RemovePersonInfoType
        //                   ,          -     CheckoutEditor
        //             PersonInfoType,         , BillingInfo, ShippingInfo,
        //  CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
        //             ), ShippingMethodInfo (              ).
        //       ShippingModule'                                                 ,
        //                                     .
        //                    ,                 PersonInfoType          .
        //
        //  PersonInfoType.
        modApiFunc('EventsManager',
               'addEventHandler',
               'RemovePersonInfoTypeEvent',
               'Shipping_Module_InterShipper',
               'OnRemovePersonInfoType');
    }

    /**
     *                    RemovePersonInfoType.
     *                  ,          -     CheckoutEditor
     *            PersonInfoType,         , BillingInfo, ShippingInfo,
     * CustomerInfo, CreditCardInfo, BankAccountInfo, PaymentMethodInfo (
     *            ), ShippingMethodInfo (              ).
     *      ShippingModule'                                                 ,
     *                                    .
     *                   ,                 PersonInfoType          .
     *
     * PersonInfoType.
     *
     *        InterShipper                         PersonInfoType:
     * shippingInfo
     *
     * @param string $person_info_type_tag - tag      person info type' ,
     *                      (turn off)   CheckoutFormEditor' .
     *
     * @return mixed - NULL,                       - "          ",       -
     *                                     ,            ,
     *        (                                   ).
     */
    function OnRemovePersonInfoType($person_info_type_tag)
    {
        global $application;

        if($this->isActive() === false)
        {
            //                        -                                .
            $value =  NULL;
        }
        else
        {
            $this->MessageResources = &$application->getInstance('MessageResources',"shipping-module-intershipper-messages", "AdminZone");
            $msg = $this->MessageResources->getMessage('MODULE_SHIPPING_INTERSHIPPER_API_MSG_REMOVE_PERSON_INFO_TYPE');

            $value = NULL;
            switch($person_info_type_tag)
            {
                case "shippingInfo":
                {
                    $value = $msg;
                    break;
                }
                default:
                {
                    break;
                }
            }
        }
        return $value;
    }


    /**
     * @return true if module is active or false if is not
     */
    function isActive()
    {
        global $application;
        $active_modules = modApiFunc("Checkout","getActiveModules", "shipping");
        return array_key_exists($this->getUid(), $active_modules);
    }

    /**
     * Uninstall the module.
     */
    function uninstall()
    {
        $query = new DB_Table_Delete(Shipping_Module_InterShipper::getTables());
        global $application;
        $application->db->getDB_Result($query);
    }

    /**
     * @param $method_id ID of shipping method of this module
     * @return true if ID is valid or false if it is not
     */
    function isValidShippingMethodId($method_id)
    {
        global $application;
        $tables=$this->getTables();
        $methods_table=$tables['sm_intershipper_methods']['columns'];

        $check_result = execQuery('SELECT_SM_INTERSHIPPER_METHODS',array('ApiClassName' => __CLASS__, 'method_id' => $method_id));

        if(isset($check_result[0]['id']))
            return true;
        else
            return false;
    }

    /**
     * @param $method_id ID of shipping method of this module
     * @param $and_calc true if it is necessary to calculate the shipping cost
     *         or false if it is not
     * @return array with the shipping method info (id,method_name,method_code,
     *         destination,available,cost)
     */
    function getShippingMethodInfo($method_id,$and_calc=false)
    {
        global $application;
        $tables=$this->getTables();
        $methods_table=$tables['sm_intershipper_methods']['columns'];

        $select_result = execQuery('SELECT_SM_INTERSHIPPER_METHODS_EXT',array('ApiClassName' => __CLASS__, 'method_id' => $method_id));

        $method=$select_result[0];
        $method["cost"]=PRICE_N_A;

        if($and_calc!=false)
        {
            //$ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
            //$pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
            $ShippingInfo = modApiFunc("Shipping_Cost_Calculator","getShippingInfo");
            $pak_weight = modApiFunc("Shipping_Cost_Calculator","getPackWeight");
            $rate=$this->getRateFromCache($ShippingInfo,$pak_weight,$method_id);
            if($rate!=false)
                $method["cost"]=$rate;
        };

        return $method;

    }

    /**
     * @return structure of the tables of this module
     */
    function getTables()
    {
        static $tables;

        if (is_array($tables))
        {
            return $tables;
        }

        $tables = array ();

        $settings = 'sm_intershipper_settings';
        $tables[$settings] = array();
        $tables[$settings]['columns'] = array
            (
                'id'                => $settings.'.sm_is_setting_id'
               ,'key'               => $settings.'.sm_is_setting_key'
               ,'value'             => $settings.'.sm_is_setting_value'
            );
        $tables[$settings]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'key'               => DBQUERY_FIELD_TYPE_CHAR50
               ,'value'             => DBQUERY_FIELD_TYPE_CHAR255
            );
        $tables[$settings]['primary'] = array
            (
                'id'
            );

        $methods = 'sm_intershipper_methods';
        $tables[$methods] = array();
        $tables[$methods]['columns'] = array
            (
                'id'                => $methods.'.sm_is_method_id'
               ,'carrier_code'      => $methods.'.sm_is_carrier_code'
               ,'method_code'       => $methods.'.sm_is_method_code'
               ,'method_name'       => $methods.'.sm_is_method_name'
               ,'destination'       => $methods.'.sm_is_method_destination'
               ,'available'         => $methods.'.sm_is_method_available'
            );
        $tables[$methods]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'carrier_code'      => DBQUERY_FIELD_TYPE_CHAR5
               ,'method_code'       => DBQUERY_FIELD_TYPE_CHAR20
               ,'method_name'       => DBQUERY_FIELD_TYPE_CHAR255
               ,'destination'       => DBQUERY_FIELD_TYPE_CHAR1 .' default \'L\''
               ,'available'         => DBQUERY_FIELD_TYPE_CHAR1 .' default \'N\''
            );
        $tables[$methods]['primary'] = array
            (
                'id'
            );

        $rates_cache = 'sm_intershipper_rates_cache';
        $tables[$rates_cache] = array();
        $tables[$rates_cache]['columns'] = array
            (
                'id'                => $rates_cache.'.sm_is_crate_id'
               ,'hash'              => $rates_cache.'.sm_is_crate_hash'
               ,'method_id'         => $rates_cache.'.sm_is_crate_method_id'
               ,'rate'              => $rates_cache.'.sm_is_crate_rate'
               ,'expire'            => $rates_cache.'.sm_is_crate_expire'
            );
        $tables[$rates_cache]['types'] = array
            (
                'id'                => DBQUERY_FIELD_TYPE_INT .' NOT NULL auto_increment'
               ,'hash'              => DBQUERY_FIELD_TYPE_CHAR50
               ,'method_id'         => DBQUERY_FIELD_TYPE_INT
               ,'rate'              => DBQUERY_FIELD_TYPE_DECIMAL20_5
               ,'expire'            => DBQUERY_FIELD_TYPE_INT
            );
        $tables[$rates_cache]['primary'] = array
            (
                'id'
            );


        global $application;
        return $application->addTablePrefix($tables);

    }

    /**
     *
     */
    function clearSettingsInDB()
    {
        global $application;
        $query = new DB_Delete('sm_intershipper_settings');
        $application->db->getDB_Result($query);
    }

    /**
     * @return array of the settings of this module
     */
    function getSettings()
    {
        $result = execQuery('SELECT_PM_SM_SETTINGS',array('ApiClassName' => __CLASS__, 'settings_table_name' => 'sm_intershipper_settings'));
        $this->Settings = array();
        for ($i=0; $i<sizeof($result); $i++)
        {
            $this->Settings[$result[$i]['set_key']] = unserialize($result[$i]['set_value']);
        }
        return $this->Settings;
    }

    /**
     * Sets settings for this module.
     *
     * @param $Settings array of the all settings of this module
     */
    function setSettings($Settings)
    {
        global $application;
        $this->clearSettingsInDB();
        $tables = $this->getTables();
        $columns = $tables['sm_intershipper_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Insert('sm_intershipper_settings');
            $query->addInsertValue($key, $columns['key']);
            $query->addInsertValue(serialize($value), $columns['value']);
            $application->db->getDB_Result($query);

            $inserted_id = $application->db->DB_Insert_Id();
        }
    }

    /**
     * Updates settings for this module.
     *
     * @param $Settings array of the updated settings of this module
     */
    function updateSettings($Settings)
    {

        global $application;
        $tables = $this->getTables();
        $columns = $tables['sm_intershipper_settings']['columns'];

        foreach($Settings as $key => $value)
        {
            $query = new DB_Update('sm_intershipper_settings');
            $query->addUpdateValue($columns['value'], serialize($value));
            $query->WhereValue($columns['key'], DB_EQ, $key);
            $application->db->getDB_Result($query);
        }

    }

    /**
     * @param $type type of the requested methods (ALL|AVAILABLE|NOT_AVAILABLE)
     * @param $and_calc true if it is necessary to calculate the shipping cost
     *        or false if it is not
     * @return array of the shipping methods arrays, every will be as
     *        (id,method_name,method_code,destination,available,cost)
     */
    function getShippingMethods($type="ALL",$and_calc=false)
    {
        global $application;

        $available_condition=($type=="AVAILABLE"?"'Y'":($type=="NOT_AVAILABLE"?"'N'":""));

        $tables=$this->getTables();

        $methods_table=$tables['sm_intershipper_methods']['columns'];

        $methods = execQuery('SELECT_SM_INTERSHIPPER_METHODS_BY_AVAILABILITY',array('ApiClassName' => __CLASS__, 'available_condition' => $available_condition));

        if($and_calc!=false and !empty($methods))
        {
            /*
            if($this->is_debug==true)
            {
                $ShippingInfo = modApiFunc("Shipping_Tester","getShippingInfo");
                $pak_weight=modApiFunc("Shipping_Tester","getPackWeight");
            }
            else
            {
                $ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
                $pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
            }
            */
            $ShippingInfo = modApiFunc("Shipping_Cost_Calculator","getShippingInfo");
            $pak_weight=modApiFunc("Shipping_Cost_Calculator","getPackWeight");

            $_rates=array();
            $cmethods=array();

            if($ShippingInfo["isMet"]!=false)
            {
                $_rates=$this->calculate_rates($ShippingInfo,$pak_weight);
            };

            if(!empty($_rates))
            {
                foreach($_rates as $service => $sdata)
                {
                    foreach($methods as $key => $method)
                    {
                            if($method['method_code']==$service)
                                $cmethods[]=array_merge($method,$sdata);
                    };
                };
            };

            $methods=$cmethods;

            $this->saveRatesToCache($ShippingInfo,$pak_weight,$methods);
        };

        return $methods;
    }

    /**
     * @param $update_array array if the IDs of the available methods,
     *      all other methods will be set to 'not available'
     */
    function updateShippingMethods($update_array)
    {
        global $application;

        $tables=$this->getTables();
        $methods_table=$tables['sm_intershipper_methods']['columns'];

        $query = new DB_Update('sm_intershipper_methods');
        $query->addUpdateValue($methods_table['available'],"N");
        $application->db->getDB_Result($query);

        if(count($update_array)==0 or !is_array($update_array))
            return;

        $query = new DB_Update('sm_intershipper_methods');
        $query->addUpdateValue($methods_table['available'],"Y");
        $query->Where($methods_table['id'], DB_IN, "('".implode("','",array_keys($update_array))."')");
        $application->db->getDB_Result($query);

        return;
    }

    /**
     * @param $s_info shipping info of the destination
     * @param $weight weight of the package
     * @param $rates array of the shipping rates for saving
     */
    function saveRatesToCache($s_info,$weight,$rates)
    {
        if($s_info["isMet"]==false or empty($rates))
            return;

        $s_country=modApiFunc("Location","getCountry",$s_info["validatedData"]["Country"]["value"]);
        $s_zipcode=$s_info["validatedData"]["Postcode"]["value"];
        $s_name = isset($s_info["validatedData"]["Firstname"]["value"]) ? $s_info["validatedData"]["Firstname"]["value"] : '';
        $s_name.= isset($s_info["validatedData"]["Lastname"]["value"])  ? $s_info["validatedData"]["Lastname"]["value"]  : '';

        $hash=md5($s_country.$s_name.$s_zipcode.$weight);

        $tables=$this->getTables();
        $rc_table=$tables['sm_intershipper_rates_cache']['columns'];

        global $application;

        $query = new DB_Delete('sm_intershipper_rates_cache');
        $query->Where($rc_table['hash'], DB_EQ, "'$hash'");
        $application->db->getDB_Result($query);

        $query = new DB_Delete('sm_intershipper_rates_cache');
        $query->Where($rc_table['expire'], DB_LT, time());
        $application->db->getDB_Result($query);

        foreach($rates as $key => $rate_info)
        {
            $query = new DB_Insert('sm_intershipper_rates_cache');
            $query->addInsertValue($hash, $rc_table["hash"]);
            $query->addInsertValue($rate_info["id"],$rc_table["method_id"]);
            $query->addInsertValue($rate_info["cost"],$rc_table["rate"]);
            $query->addInsertValue(time()+18000,$rc_table["expire"]);
            $application->db->getDB_Result($query);
        };

        return;
    }

    /**
     * @param $s_info shipping info of the destination
     * @param $weight weight of the package
     * @param $method_id ID of the required shipping method
     * @return cost of the shipping from the cache
     *      or false if the requested cost is not saved in the cache
     */
    function getRateFromCache($s_info,$weight,$method_id)
    {
        if($s_info["isMet"]==false or !is_numeric($method_id))
            return false;

        $s_country=modApiFunc("Location","getCountry",$s_info["validatedData"]["Country"]["value"]);
        $s_zipcode=$s_info["validatedData"]["Postcode"]["value"];
        $s_name = isset($s_info["validatedData"]["Firstname"]["value"]) ? $s_info["validatedData"]["Firstname"]["value"] : '';
        $s_name.= isset($s_info["validatedData"]["Lastname"]["value"])  ? $s_info["validatedData"]["Lastname"]["value"]  : '';

        $hash=md5($s_country.$s_name.$s_zipcode.$weight);

        $tables=$this->getTables();
        $rc_table=$tables['sm_intershipper_rates_cache']['columns'];

        $cached = execQuery('SELECT_SM_INTERSHIPPER_METHODS_CACHED',array('ApiClassName' => __CLASS__, 'hash' => $hash, 'method_id' => $method_id));

        if(isset($cached[0]["rate"]))
            return $cached[0]["rate"];
        else
            return false;
    }

    /**
     * @param $ShippingInfo shipping info of the destination
     * @param $pak_weight weight of the package
     * @return array of the shipping rates arrays
     */
    function calculate_rates($ShippingInfo, $pak_weight)
    {
        global $application;

        if($this->is_debug==true)
            $this->_debug_info[]=array("Calculate Type","Real-Time");

        $_rates = array();

        # get module settings
        $Settings=$this->getSettings();

        # get the origination and destination address
        $org_name  = modApiFunc("Configuration","getTagValue","StoreOwnerName");
        $org_addr1 = modApiFunc("Configuration","getTagValue","StoreOwnerStreetLine1");
        $org_addr2 = modApiFunc("Configuration","getTagValue","StoreOwnerStreetLine2");
        $org_city  = modApiFunc("Configuration","getTagValue","StoreOwnerCity");
        $org_state = modApiFunc("Location","getStateCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_STATE));
        $org_zip   = modApiFunc("Configuration","getTagValue","StoreOwnerPostcode");
        $org_cntr  = modApiFunc("Location","getCountryCode",modApiFunc("Configuration","getValue",SYSCONFIG_STORE_OWNER_COUNTRY));

        $dst_name  = isset($ShippingInfo["validatedData"]["Firstname"]["value"])?$ShippingInfo["validatedData"]["Firstname"]["value"]:'';
        $dst_name .= " ";
        $dst_name .= isset($ShippingInfo["validatedData"]["Lastname"]["value"])?$ShippingInfo["validatedData"]["Lastname"]["value"]:'';

        $dst_addr1 = isset($ShippingInfo["validatedData"]["Streetline1"]["value"]) ? $ShippingInfo["validatedData"]["Streetline1"]["value"] : '';
        $dst_addr2 = isset($ShippingInfo["validatedData"]["Streetline2"]["value"]) ? $ShippingInfo["validatedData"]["Streetline2"]["value"] : '';
        $dst_city  = isset($ShippingInfo["validatedData"]["City"]["value"])        ? $ShippingInfo["validatedData"]["City"]["value"]        : '';

        $stateId = modApiFunc("Checkout", "getPrAttr", "shippingInfo", "Statemenu");
        $dst_state = (($stateId === NULL) ? "" : modApiFunc("Location","getStateCode", $stateId));
        $dst_zip   = $ShippingInfo["validatedData"]["Postcode"]["value"];
        $dst_cntr  = modApiFunc("Location","getCountryCode",$ShippingInfo["validatedData"]["Country"]["value"]);

        # Calculate weight
        $local_wcoeff = modApiFunc("Localization","getValue","WEIGHT_COEFF");
        $weight_in_lbs = round(($pak_weight * $local_wcoeff * 1000 / 453.6),1);

        if($weight_in_lbs < 1)
            $weight_in_lbs = 1;

        # Get the declared value of package
        # Get the declared value of package
        /*
        if($this->is_debug==true)
            $decl_value = modApiFunc("Shipping_Tester","getCartSubtotal");
        else
            $decl_value = modApiFunc("Checkout", "getOrderPrice", "Subtotal");
        */
        $decl_value = modApiFunc("Shipping_Cost_Calculator","getCartSubtotal");

        # calculate the midnight of today
        $today = getdate();
        $today_midnight = mktime(0,0,0,$today['mon'],$today['mday'],$today['year']);

        # Calculate the shipping date - today + 1 day
        $ship_date=date("m/d/Y",time()+60*60*24*1);

        if($this->is_debug==true)
        {
            $this->_debug_info[]=array("Destination Country Code",$dst_cntr);
            $this->_debug_info[]=array("Destination Zip",$dst_zip);
            $this->_debug_info[]=array("Delivery type",$Settings["DELIVERY_TYPE"]=="RES"?"Residental":"Commercial");
            $this->_debug_info[]=array("Service URL is",$application->getAppIni('INTERSHIPPER_SERVER_URL'));
        }

        loadCoreFile('bouncer.php');
        $bnc = new Bouncer();
        $bnc->setMethod("POST");
        $bnc->setHTTPversion($application->getAppIni('INTERSHIPPER_HTTP_PROTOCOL_VERSION'));
        $bnc->setURL($application->getAppIni('INTERSHIPPER_SERVER_URL'));

        # generate the XML-request
        $post_params=array(
            "Version" => "2.0.0.0",
            "ShipmentID" => "",
            "QueryID" => _ml_substr(md5(time()),0,8),
            "Username" => $Settings["USERNAME"],
            "Password" => $Settings["PASSWORD"],
            "TotalCarriers" => "5",
                "CarrierCode1" => "DHL",
                    "CarrierAccount1" => "",
                    "CarrierInvoiced1" => "0",
                "CarrierCode2" => "FDX",
                    "CarrierAccount2" => "",
                    "CarrierInvoiced2" => "0",
                "CarrierCode3" => "UPS",
                    "CarrierAccount3" => "",
                    "CarrierInvoiced3" => "0",
                "CarrierCode4" => "USP",
                    "CarrierAccount4" => "",
                    "CarrierInvoiced4" => "0",
                "CarrierCode5" => "ARB",
                    "CarrierAccount5" => "",
                    "CarrierInvoiced5" => "0",
            "TotalClasses" => "4",
                "ClassCode1" => "1DY",
                "ClassCode2" => "2DY",
                "ClassCode3" => "3DY",
                "ClassCode4" => "GND",
            "DeliveryType" => $Settings["DELIVERY_TYPE"],
            "ShipMethod" => $Settings["PICKUP_TYPE"],
            "OriginationName" => $org_name,
                "OriginationAddress1" => $org_addr1,
                "OriginationAddress2" => $org_addr2,
                "OriginationAddress3" => "",
                "OriginationCity" => $org_city,
                "OriginationState" => $org_state,
                "OriginationPostal" => $org_zip,
                "OriginationCountry" => $org_cntr,
            "DestinationName" => $dst_name,
                "DestinationAddress1" => $dst_addr1,
                "DestinationAddress2" => $dst_addr2,
                "DestinationAddress3" => "",
                "DestinationCity" => $dst_city,
                "DestinationState" => $dst_state,
                "DestinationPostal" => $dst_zip,
                "DestinationCountry" => $dst_cntr,
            "Currency" => "USD",
            "ShippingDate" => $ship_date,
            "SortBy" => "Carrier",
            "TotalPackages" => "1",
                "BoxID1" => "BOX_1",
                    "Weight1" => $weight_in_lbs,
                    "WeightUnit1" => "LB",
                    "Length1" => $Settings["PK_LENGTH"],
                    "Width1" => $Settings["PK_WIDTH"],
                    "Height1" => $Settings["PK_HEIGHT"],
                    "DimensionalUnit1" => "IN",
                    "Packaging1" => "BOX",
                    "Contents1" => "OTR",
                    "Cod1" => "",
                    "Insurance1" => $decl_value,
            "TotalOptions" => "0"
        );

        $bnc->setPOSTstring($bnc->prepareDATAstring($post_params));

        if($this->is_debug==true)
            $this->_debug_info[]=array("POST params",$post_params);

        # receive the response from the InterShipper server
        $res = $bnc->RunRequest();

        if($res!=false) # if we have a response
        {
            # get XML-response
            $xml_response = $res["body"];

            if($this->is_debug==true)
                $this->_debug_info[]=array("XML Response",$xml_response);

            # parse XML-response
            loadCoreFile('obj_xml.php');
            $parser = new xml_doc(trim($xml_response));
            $parser->parse();

            foreach($parser->xml_index[0]->children as $k => $sChild)
                if($sChild->name=="PACKAGE") # get Package
                {
                    foreach($sChild->children as $k_pk => $pkChild)
                        if($pkChild->name=="QUOTE") # get Quote
                        {
                            $_service=""; # service code
                            $_rate=0;     # service cost
                            $_days=0;     # days of delivery
                            foreach($pkChild->children as $k_q => $qChild)
                                if($qChild->name=="SERVICE") # get Service code
                                {
                                    foreach($qChild->children as $k_s => $srvChild)
                                        if($srvChild->name=="CODE")
                                            $_service=$srvChild->contents;
                                }
                                elseif($qChild->name=="RATE") # get Service cost
                                {
                                    foreach($qChild->children as $k_r => $rtChild)
                                        if($rtChild->name=="AMOUNT")
                                            $_rate=$rtChild->contents/100; # diving on 100 because AMOUNT in cents
                                }
                                elseif($qChild->name=="GUARANTEEDARRIVAL") # get Guaranteed arrival date
                                {
                                    foreach($qChild->children as $k_g => $gaChild)
                                        if($gaChild->name=="DATE")
                                        {
                                            $da=explode("/",$gaChild->contents);
                                            $dd_midnight = mktime(0,0,0,$da[0],$da[1],$da[2]);
                                            $_days = ceil(($dd_midnight - $today_midnight) / 86400);
                                        }
                                }
                                elseif($qChild->name=="NONGUARANTEEDARRIVAL" and $_days==0) # get arrival interval
                                {
                                    $earliest=0;
                                    $latest=0;
                                    foreach($qChild->children as $k_n => $ngaChild)
                                        if($ngaChild->name=="EARLIESTARRIVALDATE")  # get earliest day
                                        {
                                            $da=explode("/",$ngaChild->contents);
                                            $dd_midnight = mktime(0,0,0,$da[0],$da[1],$da[2]);
                                            $earliest = ceil(($dd_midnight - $today_midnight) / 86400);
                                        }
                                        elseif($ngaChild->name=="LATESTARRIVALDATE") # get latest day
                                        {
                                            $da=explode("/",$ngaChild->contents);
                                            $dd_midnight = mktime(0,0,0,$da[0],$da[1],$da[2]);
                                            $latest = ceil(($dd_midnight - $today_midnight) / 86400);
                                        };
                                    $_days = "$earliest - $latest";
                                };
                            # put a rate into the rates array
                            $_rates[$_service]=array("cost"=>$_rate,"days"=>$_days);
                        };
                }
                elseif($sChild->name=="ERROR" and $this->is_debug==true)
                    $this->_debug_info[]=array("Error",$sChild->contents);
        }
        elseif($this->is_debug==true)
            $this->_debug_info[]=array("Error",$bnc->getLastErrorMessage());

        return $_rates;
    }

    function setDebugFlag($flag=true)
    {
        if(is_bool($flag))
            $this->is_debug=$flag;
    }

    function getDebugInfo()
    {
        return $this->_debug_info;
    }

    /**#@-*/

//------------------------------------------------
//              PRIVATE DECLARATION
//------------------------------------------------

    /**#@+
     * @access private
     */

    /**
     * @param $method_id ID of shipping method of this module
     * @return shipping cost or PRICE_N_A
     */
    function getShippingCost($method_id="")
    {
        $ShippingInfo = modApiFunc("Checkout", "getPrerequisiteValidationResults", "shippingInfo");
        $pak_weight=modApiFunc("Checkout", "getOrderWeight", "netto");
        $rate=$this->getRateFromCache($ShippingInfo,$pak_weight,$method_id);
        if($rate!=false)
            return $rate;
        else
            return PRICE_N_A;
    }

    var $bActive;

    var $is_debug;
    var $_debug_info;

    /**#@-*/
}
?>